@@ -6,19 +6,19 @@ from .parsing import ignored, flag
 from .utils import print_args
 
 
-def extract_parameters(f, ignore=["self"]):
+def read_parameters(f, ignore=["self"]):
     params = inspect.signature(f).parameters.values()
     params = [p for p in params if p.name not in ignore]
     return params
 
 
-def parse_args_from_cli(f, self):
+def read_kwargs_from_cli(f, runner):
     empty = inspect.Parameter.empty
-    params = extract_parameters(f)
+    params = read_parameters(f)
 
     for p in params:
-        if hasattr(self.args, p.name):
-            raise argparse.ArgumentError(f"{p.name} conflicts with exsiting args.")
+        if hasattr(runner.args, p.name):
+            raise TypeError(f"{p.name} conflicts with exsiting args.")
 
         annotation = p.annotation
         if annotation is empty:
@@ -34,123 +34,122 @@ def parse_args_from_cli(f, self):
 
         if annotation is flag:
             if p.default is empty:
-                raise TypeError(f"Flag {p.name} should have a default value.")
-            self.add_argument(f"--{p.name}", action="store_true", default=p.default)
+                p.default = False
+            runner.add_argument(f"--{p.name}", action="store_true", default=p.default)
         elif p.default is empty:
-            self.add_argument(f"{p.name}", type=annotation)
+            runner.add_argument(f"{p.name}", type=annotation)
         else:
-            self.add_argument(f"--{p.name}", type=annotation, default=p.default)
+            runner.add_argument(f"--{p.name}", type=annotation, default=p.default)
 
     # parse args from cli
-    self.parse_args(strict=True)
+    runner.parse_args(strict=True)
 
-    # args attached explicitly to the wrapped.args, and passed to f
-    explicit = {
-        p.name: getattr(self.args, p.name, p.default)
+    if runner.verbose:
+        p_names = {p.name for p in params}
+        global_args = {k: v for k, v in vars(runner.args).items() if k not in p_names}
+        command_args = {k: v for k, v in vars(runner.args).items() if k in p_names}
+        print_args(global_args, command_args)
+
+    return {
+        p.name: getattr(
+            runner.args,
+            p.name,
+            p.default,
+        )
         for p in params
-        if p.annotation is not ignored
     }
 
-    # args passed to f without attaching to wrapped.args
-    implicit = {}
 
-    # remove parsed args from self.args
-    for p in params:
-        if p.annotation is not ignored:
-            delattr(self.args, p.name)
-
-    return explicit, implicit
-
-
-def parse_args_from_call(f, self, *args, **kwargs):
+def read_kwargs_from_call(f, runner, *args, **kwargs):
     """
     Call as function, use passed parameters to update args.
     """
-    params = extract_parameters(f)
+    empty = inspect.Parameter.empty
+    params = read_parameters(f)
 
     # args => kwargs
     for p, a in zip(params, args):
         kwargs[p.name] = a
-    del args
 
-    explicit = {
-        p.name: kwargs.get(
-            p.name,
-            p.default,
-        )
-        for p in params
-        if p.annotation is not ignored
-    }
+    # default => kwargs
+    for p in params:
+        if p.name not in kwargs and p.default != empty:
+            kwargs[p.name] = p.default
 
-    implicit = {
-        p.name: kwargs.get(
-            p.name,
-            p.default,
-        )
-        for p in params
-        if p.annotation == ignored
-    }
+    # update runner.args
+    for p in params:
+        if p.annotation != ignored:
+            setattr(runner.args, p.name, kwargs.get(p.name, p.default))
 
-    return explicit, implicit
+    return kwargs
 
 
-def inherit_signature(f, g):
-    """
-    Args:
-        f: dervied member function
-        g: base member function
-    """
-    f_params = inspect.signature(f).parameters.values()
-    f_params = [p for p in f_params if p.name not in ["args", "kwargs"]]
-    names = set([p.name for p in f_params])
-    g_params = inspect.signature(g).parameters.values()
-    g_params = [p for p in g_params if p.name not in names]
+class _Command:
+    def __init__(self, f, inherit):
+        self.f = f
+        self.inherit = inherit
 
-    for i, p in enumerate(f_params):
-        if p.kind is not p.POSITIONAL_OR_KEYWORD:
-            i -= 1
-            break
+    def _inherit_signature(self, owner, name):
+        """
+        Args:
+            f: dervied member function
+            g: base member function
+        """
+        f = self.f
+        g = getattr(super(owner, owner), name, None)
 
-    for p in g_params:
-        if p.kind == p.POSITIONAL_OR_KEYWORD:
-            f_params.insert(i + 1, p)
-        else:
-            f_params.append(p)
+        if g is None:
+            raise TypeError(
+                f'Cannot inherit method "{name}" from the parent class of "{owner.__name__}".'
+            )
 
-    f.__signature__ = inspect.Signature(f_params)
+        f_params = inspect.signature(f).parameters.values()
+        f_params = [p for p in f_params if p.name not in ["args", "kwargs"]]
+        p_names = set([p.name for p in f_params])
+        g_params = inspect.signature(g).parameters.values()
+        g_params = [p for p in g_params if p.name not in p_names]
 
-    return f
+        for i, p in enumerate(f_params):
+            if p.kind is not p.POSITIONAL_OR_KEYWORD:
+                i -= 1
+                break
 
+        for p in g_params:
+            if p.kind == p.POSITIONAL_OR_KEYWORD:
+                f_params.insert(i + 1, p)
+            else:
+                f_params.append(p)
+
+        f.__signature__ = inspect.Signature(f_params)
 
-class command:
-    def __init__(self, f):
         self.f = f
 
     def __set_name__(self, owner, name):
+        if self.inherit:
+            self._inherit_signature(owner, name)
+
         f = self.f
 
-        parent = getattr(super(owner, owner), name, None)
-        if parent is not None:
-            inherit_signature(f, parent)
-
         @functools.wraps(f)
-        def wrapped(self, *args, **kwargs):
+        def wrapped(runner, *args, **kwargs):
             if kwargs.get("_call_as_command", False):
                 assert len(args) == 0 and len(kwargs) == 1
-                explicit, implicit = parse_args_from_cli(f, self)
-                wrapped.args = argparse.Namespace(**explicit)
-                if self.verbose:
-                    print_args(self.args, wrapped.args)
+                kwargs = read_kwargs_from_cli(f, runner)
             else:
-                explicit, implicit = parse_args_from_call(f, self, *args, **kwargs)
-                wrapped.args = argparse.Namespace(**explicit)
-            return f(self, **explicit, **implicit)
+                kwargs = read_kwargs_from_call(f, runner, *args, **kwargs)
+            return f(runner, **kwargs)
 
         wrapped.is_command = True
 
         setattr(owner, name, wrapped)
 
 
+def command(f=None, inherit=False):
+    if f is None:
+        return lambda f: _Command(f, inherit)
+    return _Command(f, inherit)
+
+
 def possible_commands(obj):
     commands = []
     for name in dir(obj):
@@ -167,6 +166,7 @@ def possible_commands(obj):
 class Runner:
     def __init__(self, verbose=False):
         self.verbose = verbose
+        self.parse_args()
 
     @property
     def parser(self):
@@ -199,9 +199,6 @@ class Runner:
         self.command = self.args.command
         return self.args
 
-    def update_args(self, args):
-        self.args = argparse.Namespace(**{**vars(self.args), **vars(args)})
-
     def run(self):
         command = getattr(self, self.command)
         command(_call_as_command=True)
